%% Model to simulate water flow and soil moisture dynamcis using the 1D Richards approach
% Code for task 2 to see soil moisture response of different
% soils and explore model sensitivity

clear all;
close all;

fid=fopen('richards_input_task2.dat'); % opens main control file

%% Initialisation with overview and preallocation of important variables

z=[0:-0.025:-1.5]'; % defines vertical extent of 1D soil domain and soil domain grid size
depth=[0 -0.025 -0.05 -0.10 -0.2]'; % points for plotting soil moisture time series
dim=length(z);
dz= abs(diff(z)); % grid size
dim=length(z); % length of vector with grid nodes
istyp=ones(dim,1); % pointer on soil types
psi=zeros(dim,1); % Matric potential
th_star=zeros(dim,1); % relative saturation
theta=zeros(dim,1); % soil moisture
theta_new=zeros(dim,1); % soil moisture at new time level
psinew=zeros(dim,1); % matrix potential at new state
c=zeros(dim,1); % water capacity
k=zeros(dim,1);  % soil hydaulic conductivity  
q=zeros(dim-1,1); % water flux in the soil  
qgeo=zeros(dim-1,1);
theta_ar=[]; % array containing soil moisture from richards eq. to be dumped in file theta_out_richards
q_ar=[]; % array for plotting of darcy fluxes
time_v=[]; % array containing simulation times

%--------------------------------------------------------------------------
% the following variables containing the file paths to the input data

inistate_file=fgetl(fid); %contains initial states
soilpara_file=fgetl(fid); %contains soil hydraulic parameters and depth assigment
precip_file=fgetl(fid); %contains precipitation data
etp_file=fgetl(fid); %contains potential evoporation data
i_inits=fgetl(fid); %defines type of initial state
t_ends=fgetl(fid); %end of time loop
dtcs=fgetl(fid); %  time step control for courant criterion
fclose(fid);

% convert to integer
i_init=sscanf(i_inits(1), '%i');
t_end=sscanf(t_ends, '%i');
dt=sscanf(dtcs(1:4), '%f');

%% Read input data

% read precipitation data from file
[prec_c]=read_precip_new(precip_file); 
prec=prec_c(:,2);

% read ETP data from file
[etp_c]=read_etp_new(etp_file);
etp=etp_c(:,2);

% read soil types from a file in array para 
[thr, ths, alpha, n_vg, ks, stor, istyp, l_vg]=soilpara(soilpara_file,dim,z);

% read intial theta and psi states from file in array init
[theta_initial, psi]=readini_state(inistate_file, i_init,z,dim);
theta = theta_initial;

%% Calculation and Conversion of initial hydraulic properties (theta, psi, k)

%calculate psi from theta or theta from psi 
if i_init==1
  [psi, c]=psi_theta(theta,thr,ths,alpha,n_vg,ks,stor,istyp,l_vg,dim,z);
  psinew=psi;
else 
   psinew=psi;
  [theta, c]=theta_psi(psi,thr,ths,alpha,n_vg,ks,stor,istyp,l_vg,dim,z);  
end

% calculate hydraulic conductivity
k=k_psi(psinew,thr,ths,alpha,n_vg,ks,stor,istyp,l_vg,dim,z);


%% Definition of time and initial upper and lower boundary conditions of soil domain

% set starttime
time=prec_c(1,1);
t_end=t_end+time;
time_v=[];
t_boundary=prec_c(:,1); % timesteping of boundary data
i_time=1; % time step counter

% upper boundary
psipot=0;
psi_u=psipot;
qb_u=-0.5*(prec(1)+prec(2)); % potential flux at upper boundary calculate from Precipitation

if theta(1) >= ths(istyp(1))
    ibtyp_u=2;
    psinew=psipot;
else 
    ibtyp_u=1;
end

% lower boundary
ibtyp_l=1; 
qb_l= -(k(dim-1)+ks)/2; % flux condition at lower boundary

%% Start of Richards solver and calculation of water flow and soil moisture change
figure;

while time < t_end
    
% --- solve Richards equation  
   phi=psi+z; % calculate hydraulic potential
   
   %compute Darcy water fluxes at inner nodes of soil domain
   q=darcy(phi,k,dim,dz);

   % Predictor step (calculates soil moisture change for a half timestep --> intermediate state) 
   theta_new=euler_th(theta,q,ibtyp_u,qb_u,ibtyp_l,qb_l,istyp,thr,ths,dz,0.5*dt,dim);
   
   % calculate new psi at intermediate state   
   [psi, c]=psi_theta(theta_new,thr,ths,alpha,n_vg,ks,stor,istyp,l_vg,dim,z);
   
   % calculate new hydraulic conductivity at intermediate state
   k=k_psi(psi,thr,ths,alpha,n_vg,ks,stor,istyp,l_vg,dim,z);
   
   % calculate new Darcy flux using k at intermediate state
   q=darcy(phi,k,dim,dz);
  
   % corrector step (calculates with the new intermediate state variables
   % the second half of the timestep and the total soil moisture change of the timestep) 
   theta_new=euler_th(theta,q,ibtyp_u,qb_u,ibtyp_l,qb_l,istyp,thr,ths,dz,dt,dim);
   
   % calculate new psi at final state
   [psi, c]=psi_theta(theta_new,thr,ths,alpha,n_vg,ks,stor,istyp,l_vg,dim,z);
    
   % calculate hydraulic conductivity at final state
   k=k_psi(psi,thr,ths,alpha,n_vg,ks,stor,istyp,l_vg,dim,z);

 %---- Update of time and boundary conditions
   time=time+dt;
   theta=real(theta_new);
 
   % Find actual position in boundary conditions time series
   ipos=max(find(t_boundary <= time));
   
   % actual flux at upper boundary, linear interpolation between 2 time steps 
   qb_u=-0.5*(prec(ipos)+prec(ipos+1));

   % surface runoff
   q_up(i_time)=qb_u-q(1); 
   if q_up(i_time) < 0
       q_up(i_time) =0;
   end
   
   % switch off upper boundary conditions in case of saturation
   if theta(1) >= ths(istyp(1)) 
       ibtyp_u=2;
       psi_u=psipot;
   else 
       ibtyp_u=1;
   end
   
   % upper and lower boundary condition
   if qb_u >= 0 
     qb_u=0.5*(etp(ipos)+etp(ipos+1));
   end
   
   if ibtyp_l==3
       qb_l=-k(dim-1);
   end
   
 %---- Plot of current soil moistures and Darcy fluxes  
   subplot(2,1,1);
   h1=plot(real(theta),z,'b-','linewidth',2);
   title(['time' num2str(time) ' s' ],'fontsize',14);
   xlabel('Theta [-]','fontsize',14);
   ylabel('z [m]','fontsize',14);
   legend(h1,'Richards');
   xlim([thr(istyp(1)) ths(istyp(1)) ]);
   set(gca,'fontsize',14,'linewidth',2);
   ylim([-1.5 0])
   
  
   subplot(2,1,2)
   h1=plot(real(q)',z(1:dim-1),'b-','linewidth',2);
   xlabel('Flux [m/s] ','fontsize',14);
   ylabel('z [m]','fontsize',14);
   legend(h1, 'Darcy flux');
   set(gca,'fontsize',14,'linewidth',2)
   ylim([-1.5 0])
   
   % Counters for plotting 
   i_time=i_time+1;
    
   MM(i_time)=getframe;
   time_v=[time_v time];
   theta_ar=[real(theta_ar) theta]; %save soil moisture from richards model to array, theta time series
   q_ar=[q_ar q']; %save darcy flux q to array

end % end of Richards solver and calculation of soil moisture change

%% Plotting of simulation results and comparison with observed data

% plot final states
figure;

%final soil moisture profiles (initial, simulated, observed)
subplot(2,1,1);
h1=plot(theta,z,'b-','linewidth',2);
hold on; 
h4=plot(theta_initial(1:dim-1),z(1:dim-1),'k--','linewidth',2);
hold off;
title(['time' num2str(time) ' s' ],'fontsize',14);
xlabel('Theta [-]','fontsize',14);
ylabel('z [m]','fontsize',14);
legend([h1 h4 ],'Richards eq.','initial');
xlim([0.2 0.35])
set(gca,'fontsize',14,'linewidth',2);
ylim([-1.5 0])

% time series of soil mositure profile for the first 20 cm 
ip=find(z > -0.2);
subplot(2,1,2);
pcolor(time_v,z(ip),real(theta_ar(ip,:)));
shading flat;
colorbar ;
caxis([min(min(theta_ar)), max(max(theta_ar))]); 
xlabel ('time [s]','fontsize',14);
ylabel ('depth [m]','fontsize',14);
set(gca,'linewidth',1,'fontsize',14);

